#!/usr/bin/sh

#
#- NNFie.sh:	This tool transports the NNSYRF formats from one database, 
#				either version 3.1, 3.2, or 4.0 to a version 4.0 database.
#
#				Users are expected to pass command-line options exactly the
#				same way they pass them to NNFie, the executable.
#
#				Ths syntax is:
#					
#				 NNFie  ((-C <command file name> 
#         				 (-i <import file name>|-e <export file name>
#          				 [-m <format name>+]
#          				 [-s <session name>]))
# 
# 				 -C = Alternate command file: default file is NNFie.cmd
#    				if this option is given, NNFie will read command-line
#    				options from a file instead of the command line.
# 				 -i = import: <import file name>: default file NNFie.exp
# 				 -e = export: <export file name>: default file NNFie.exp
# 				 -s = session name: default is nnfie
# 				 -m = message type/format: default results in export of 
#					all formats
#
#				Given the names of formats, the tool will also transport all
#				conditional-branching rules associated with them.  The export
#				output of the conditional-branching rules then will go to
#				a file named, <Formatter import/export file name>.rules.
#				
#				If -C option is turned on, NNFie.sh will generate a 
#				command file for NNRie (it will named <Formatter command 
#				file name>.rules>).  #
#
#				Users are expected to ensure that old files with the same 
#				name are not currently present when running export.
#

#=============
# global vars.
#=============

if [ `\uname` = "Windows_NT" ]
then
	EXE=".exe"
else
	EXE=""
fi

FMT_IE_CMD="NNFie$EXE"
RUL_IE_CMD="NNRie$EXE"

NNFIE_FILE="NNFie.exp"
SESSION_NAME="nnfie"
FMT_CMD_FILE="NNFie.cmd"

RUN_NNRIE=1
USE_CMD_FILE=0

#===============================================================================
# parse the 4.0 command-line options and save data for NNRie command-line 
# options.
#===============================================================================

parse_40_cmd()
{
	if [ $# -le 0 ]
	then
		return
	fi

	case $1 in
		-C )	
			USE_CMD_FILE=1
			shift
			if [ $# -eq 1 ]
			then
				FMT_CMD_FILE=$1
			else
				return
			fi
			;;
		-i )	
			IE_MODE=IMPORT
			shift

			if [ $# -gt 0 ]
			then
				if [ $1 != "-s" ]
				then
					NNFIE_FILE=$1
					shift
				fi	
			fi
			;;
		-e ) 	
			IE_MODE=EXPORT_ALL
			shift

			if [ $# -le 0 ]
			then
				return
			fi

			if [ $1 != "-m" ]
			then
				NNFIE_FILE=$1
				shift
			fi

			if [ $# -le 0 ]
			then
				return
			fi

			if [ $1 = "-m" ]
			then
			 	IE_MODE=EXPORT_BY_NAME
				shift

				if [ $# -le 0 ]
				then
					return
			 	fi

				FORMAT_LIST=""
				FORMAT_COUNT=0

				while [ $# -gt 0 ]; do
					if [ $1 = "-s" ]
					then
						break;
					else
						FORMAT_LIST="$FORMAT_LIST $1"
						shift
						FORMAT_COUNT=`expr $FORMAT_COUNT + 1`
					fi
				done

				if [ $FORMAT_COUNT -le 0 ]
				then
					return
				fi
			fi
			;;
		* )
			return
			;;
	esac

	if [ $USE_CMD_FILE -eq 0 ]
	then
		if [ $# -gt 0 ]
		then
			if [ $1 = "-s" ]
			then
				shift

				if [ $# -le 0 ]
				then
					return
				fi
	
				SESSION_NAME=$1
				shift
			fi
		fi
	
		if [ $# -gt 0 ]
		then
			return
		fi
	fi
}	# parse_40_cmd

#===============================================================================
# parse the 3.1/3.2 command-line options and save data for NNRie command-line 
# options.
#===============================================================================

parse_31_32_cmd()
{
	if [ $# -le 0 ]
	then
		return
	fi

	case $1 in
		-import )	
				IE_MODE=IMPORT
				shift
				;;
		-export )	
				if [ $# -le 0 ]; then
					return
				fi

				shift

				case $1 in
					-all ) 	
							IE_MODE=EXPORT_ALL
							shift
							;;
					-name ) 
							IE_MODE=EXPORT_BY_NAME
							shift

							if [ $# -le 0 ]
							then
								return
							fi
							
							FORMAT_LIST=""
							FORMAT_COUNT=0

							while [ $# -gt 0 ]; do 
								if [ $1 = "-file" ]
								then
									break;
								elif [ $1 = "-session" ]
								then
									break;
								else
									FORMAT_LIST="$FORMAT_LIST $1"
									FORMAT_COUNT=`expr $FORMAT_COUNT + 1`
									shift
								fi
							done

							if [ $FORMAT_COUNT -le 0 ]
							then
								return
							fi
							;;
					* )	
							return
							;;
				esac
				;;
		* )
				return
				;;
	esac
	
	if [ $# -gt 0 ]
	then
		if [ $1 = "-file" ]
		then
			shift

			if [ $# -le 0 ]
			then
				return
			fi

			NNFIE_FILE=$1
			shift
		fi
	fi

	if [ $# -gt 0 ]
	then
		if [ $1 = "-session" ]
		then
			shift

			if [ $# -le 0 ]; then
				return
			fi

			SESSION_NAME=$1
			shift
		fi
	fi

	if [ $# -gt 0 ]; then
		return
	fi
}	# parse_31_32_cmd

#-------------------------------------------------------------------------------

# if NNFie is missing, alert users and exit.

if [ ! -f $FMT_IE_CMD ]; then
	echo "NNFie.sh: NNFie is not found in the current directory."
	echo
	exit 1
fi

# if NNRie does not exist, warn users and continue. 
if [ ! -f $RUL_IE_CMD ]; then
	RUN_NNRIE=0
	echo "Warning (NNFie.sh):  NNRie does not exist in the current directory."
	echo "    Conditional branching rules will not be exported without NNRie."
	echo
fi

#===============
# execute NNFie.
#===============

# if NNFie failed, then do not run NNRie.  instead, just exit.

$FMT_IE_CMD $@

if [ $? -ne 0 ]
then
	echo "NNFie.sh: Error occurred during execution of NNFie."
	echo
	exit 1
fi

#===============================================
# execute NNRie for conditional-branching rules.
#===============================================

if [ $RUN_NNRIE -eq 1 ]
then
	# parse the command-line options to pass them to NNRie.
	if [ "$1" = "-import" -o "$1" = "-export" ]
	then
		parse_31_32_cmd $@
	else
		parse_40_cmd $@
	fi

	NNRIE_FILE=$NNFIE_FILE.rules

	if [ $USE_CMD_FILE -eq 0 ]
	then
		case $IE_MODE in
			IMPORT )
					if [ -s $NNRIE_FILE ]
					then
						$RUL_IE_CMD -i $NNRIE_FILE -f "$NNRIE_FILE.failed" \
									-s $SESSION_NAME
					fi
					;;
			EXPORT_ALL )
					$RUL_IE_CMD -e $NNRIE_FILE -s $SESSION_NAME \
								-a NNSYRF_FORMATTER
					;;
			EXPORT_BY_NAME )
					$RUL_IE_CMD -e $NNRIE_FILE -s $SESSION_NAME \
								-a NNSYRF_FORMATTER -m $FORMAT_LIST
					;;
			* )		
					echo "ERROR: invalid import/export mode!"
					exit 1
					;;
		esac
	else
	# -----------------
	# use command file.
	# -----------------

		RUL_CMD_FILE="$FMT_CMD_FILE.rules"
		TMP_RUL_CMD=`cat $FMT_CMD_FILE | tr '\012' '\040'`
		USE_CMD_FILE=0
		parse_40_cmd $TMP_RUL_CMD

		NNRIE_FILE=$NNFIE_FILE.rules

		#
		# generate a command file for NNRie.
		#
		case $IE_MODE in
			IMPORT )
					if [ -s $NNRIE_FILE ]
					then
						echo "-i $NNRIE_FILE -f $NNRIE_FILE.failed \
-s $SESSION_NAME" > $RUL_CMD_FILE
						
						$RUL_IE_CMD -C $RUL_CMD_FILE
					fi
					;;
			EXPORT_ALL )
					echo "-e $NNRIE_FILE -s $SESSION_NAME \
-a NNSYRF_FORMATTER" > $RUL_CMD_FILE

					$RUL_IE_CMD -C $RUL_CMD_FILE
					;;
			EXPORT_BY_NAME )
					echo "-e $NNRIE_FILE -s $SESSION_NAME \
-a NNSYRF_FORMATTER -m $FORMAT_LIST" > $RUL_CMD_FILE

					$RUL_IE_CMD -C $RUL_CMD_FILE
					;;
			* )		
					echo "ERROR: invalid import/export mode!"
					exit 1
					;;
		esac
	fi

	if [ $IE_MODE = "EXPORT_ALL" -o $IE_MODE = "EXPORT_BY_NAME" ]
	then
		if [ ! -s $NNRIE_FILE ]
		then
			rm -f $NNRIE_FILE
		fi
	fi
fi
